#!/usr/local/bin/perl
#
# Purpose: Converts your computer code's files (e.g., MATLAB m-files,
#          fortran and C source files) into html format for your users
#          to view them with a browser.
# mfc2html generates 3 HTML files:
#          1) The main html page, index.html, that has a 3-frame layput.
#             The top frame spans across the entire wide of the page is
#             for the user to provide a flow_diagram that depicts the flow
#             logic of the m-files in the current dir. The remaining two frames
#             are layout to share the width of the page. The two files for these
#             two frames are described below.
#          2) An html file, mfile-list.html, that lists the names of all 
#             m-files in current dir,  along with links to the respective m-files.
#          3) Generate the html equivalence of the m-files.
#          This provides the necessary html files for a 2-frame setup so that 
#          a click to a file of the list on the left frame will display the
#          the corresponding m-file on the right hand frame.
# Usage  : % mfc2html.pl [filetype] [image-file] [R   G   B]
#     All input arguments are optional
#     filetype  - one of .m, .f, c, *, or others. 
#                 Default is .m
#     imagefile - an image file to be displayed on the top of HTML page.
#                 Recommended formats are gif, jpg. 
#                 Default image file is the included 'BUlogo.gif'.
#     bgcolor   - Background RGB color for source code listing.
#                 Default is gray, [0.8, 0.8, 0.8].
# OUTPUT : All output files go to the HTML sub-directory. 
# Note   : All files in current dir that satisfy filetype will be used.
# CAUTION: An HTML sub-directory will be generated or overwritten. <======= 
# Kadin Tseng, SCV, Boston University
# April 9, 2009
############################################################################
# First, process the command line input argument. Expects the user
# to provide the image file (flow diagram) for display on the top frame.
# If not, a BU logo image file that comes with the perl script will be used.

use Cwd;
$mycwd = cwd();

$numarg = $#ARGV + 1;
if ($numarg < 2) {
  print "***********************************************************\n";
  print "*                                                         *\n";
  print "* mfc2html usage:                                         *\n";
  print "* > mfc2html.pl .m  /your-dir/imagefile [Red Green Blue]  *\n";
  print "* More details in mfc2html.pl or mfc2html_readme          *\n";
  print "*                                                         *\n";
  print "***********************************************************\n";
  exit 0;
} else {
  print "***************************************************************\n";
  print "                                                               \n";
  print "  HTML files are generated for each m-file in this dir:        \n";
  print "  $mycwd/HTML\n";
  print "  To activate the html files in the MATLAB environment:        \n";
  print "  >> open('HTML/index.html')    % for details, see readme file \n";
  print "                                                               \n";
  print "***************************************************************\n";
  $filetype = $ARGV[0];
  $imagefile = $ARGV[1];
  $Red = 0.8;  $Green = 0.8;  $Blue = 0.8;
  if ($numarg == 5) {
    $Red = $ARGV[2];
    $Green = $ARGV[3];
    $Blue = $ARGV[4];
  }
}
############################################################################
# if HTML dir exists, remove all files; else create HTML dir
if (-d "./HTML") {
  @files = glob("./HTML/*");
  foreach $file (@files) {
    unlink($file);
  }
} else {
  mkdir("HTML", 0755);   # create dir HTML if it doesn't exist
}
############################################################################
# This section deal with image files.
# Copy the image file for the top frame; do it "by hand" for portability
use File::Basename;
# acceptable image suffix types
@suffixlist = (".gif", ".jpg", ".png", ".eps", ".pdf");
($name, $path, $suffix) = fileparse($imagefile, @suffixlist);
$newimagefile = "./HTML/$name$suffix";

#$oldfile = $imagefile;
#$newfile = $newimagefile;
#mycopy($oldfile, $newfile);

use File::Copy;
copy($imagefile, $newimagefile);

###########################################################################
open(FRAMES, ">HTML/index.html"); # open and generates index.html file
print FRAMES <<"MyLabel";
<html>
<frameset rows="40%,60%">

<frame src="flow_diagram.html">

<frameset cols="25%,75%">
<frame src="mfile-list.html" name="sourcelist">
<frame src="sample-function.html" name="sourcecode">
</frameset>

</frameset>

</html>
MyLabel
###########################################################################
# Generate the initial bottom right-hand frame htmlcontent
open(SAMPLE, ">HTML/sample-function.html"); # open and generates index.html file
print SAMPLE <<"MyLabel";
<html>
<body>
<p>
<h1>Click on a function in the left frame to display it here.</h1>
<p>
</body>
</html>
MyLabel
############################################################################
# Generate the main index.html file that lays out 3 frames
open(FLOW, ">HTML/flow_diagram.html"); # open and generates index.html file
print FLOW <<"MyLabel";
<html>
<body>
<center>
<img src="../$newimagefile" border=0>
</center>
</body>
</html>
MyLabel
###########################################################################
# Generate the file list html file for the bottom left-hand frame
# $filetype is, for example, .m, .f, .c
@filename_list = glob("*$filetype");   # get list of files according to $filetype
open(LIST, ">HTML/mfile-list.html"); #open for write, append
print LIST "<html>\n<body>\n<p><h2>M-file List</h2>\n";
print LIST "<h4>(Select m-file to display on the right)</h4>\n";
print LIST "<ol>\n";
use POSIX qw(ceil floor);
# The input RGB color has range (0-1) (may come from MATLAB's uisetcolor)
# Convert it to (0-255) *integers* for use in HTML background color setting
$Red = floor($Red*255);
$Green = floor($Green*255);
$Blue = floor($Blue*255);
foreach $line (@filename_list) {
    @words = split(" ", $line);   # parse line; remove newline
    ($mfile) = @words;            # m-file is first (and only) output
    $html = $mfile;               # copy mfile to html file
    $html =~ s/\.m/\.html/;       # change suffix .m to .html
    print LIST qq[  <li><a href="$html" target="sourcecode">$mfile</a></li>\n];
    open(OUTSRC, ">HTML/$html"); #open a new html file for write, append
    open(INSRC, "$mfile") || die "Couldn't open the file !";
    print OUTSRC "<html>\n<body>\n<p>\n<h2>$mfile</h2>\n<p>\n";
    print OUTSRC qq[<div style="background:rgb($Red,$Green,$Blue)">\n<pre><code>\n];
    while (my $line_src = <INSRC>) {
	  chomp($line_src);
	  print OUTSRC "$line_src\n";
    }
    print OUTSRC "\n<p>\n</div>\n";
    print OUTSRC "\n</code></pre>\n</body>\n</html>\n";   # remaining HTML tags
    close(INSRC);
    close(OUTSRC);
}
print LIST "</ol>\n</code></pre>\n</body>\n</html>\n";   # remaining HTML tags
close(LIST);
############################################################################

sub mycopy  {
open(IN,  "< $oldfile")       or die "can't open $oldfile:  $!";
open(OUT, "> $newfile")       or die "can't open $newfile:  $!";

$blksize = (stat IN)[11] || 16384;          # preferred block size?
while ($len = sysread IN, $buf, $blksize) {
    if (!defined $len) {
        next if $! =~ /^Interrupted/;       # ^Z and fg
        die "System read error: $!\n";
    }
    $offset = 0;
    while ($len) {          # Handle partial writes.
        defined($written = syswrite OUT, $buf, $len, $offset)
            or die "System write error: $!\n";
        $len    -= $written;
        $offset += $written;
    };
}

close(IN);
close(OUT);
}
